from typing import Optional

from faststream.redis import RedisBroker, StreamSub
from pydantic import PositiveInt

from approck_messaging.models.message import TransportMessage


class Publisher:
    def __init__(self, broker: RedisBroker, stream: str, maxlen: Optional[PositiveInt] = None) -> None:
        self.broker = broker
        self.publisher = self.broker.publisher(stream=StreamSub(stream=stream, maxlen=maxlen))

    @classmethod
    def from_uri(cls, redis_uri: str, stream: str, maxlen: Optional[PositiveInt] = None) -> "Publisher":
        return cls(broker=RedisBroker(redis_uri), stream=stream, maxlen=maxlen)

    async def send_message(self, message: TransportMessage) -> None:
        await self.publisher.publish(message)



import enum
from typing import Optional, List, Dict, Any

from pydantic import BaseModel, HttpUrl


class MessageType(str, enum.Enum):
    GENERIC = "generic"
    VIDEO_NOTE = "video_note"


class MessageButton(BaseModel):
    label: str
    url: HttpUrl


class MessageMedia(BaseModel):
    id: str
    name: str
    url: HttpUrl
    type: str
    status: str


class Message(BaseModel):
    type: MessageType

    # Generic
    caption: Optional[str] = None
    media: Optional[List[MessageMedia]] = None
    buttons: Optional[List[MessageButton]] = None

    # Video Note
    video_note: Optional[MessageMedia] = None


class TransportMessageRecipient(BaseModel):
    telegram_id: int


class TransportMessage(Message):
    recipient: TransportMessageRecipient
    extra: Optional[Dict[str, Any]] = None



from typing import Union

from faststream import context
from faststream.redis import RedisBroker, StreamSub
from faststream.redis.subscriber.asyncapi import AsyncAPISubscriber


class Subscriber:
    def __init__(self, broker: RedisBroker, **kwargs) -> None:
        self.broker = broker

        for key, value in kwargs.items():
            context.set_global(key, value)

    @classmethod
    def from_uri(cls, redis_uri: str, **kwargs) -> "Subscriber":
        return cls(broker=RedisBroker(redis_uri), **kwargs)

    def message(self, stream: Union[StreamSub, str]) -> AsyncAPISubscriber:
        return self.broker.subscriber(stream=stream)
